@@ -32,7 +32,7 @@ from gajim.common.modules.message_util import get_chat_type_and_direction
 from gajim.common.modules.message_util import get_eme_message
 from gajim.common.modules.message_util import get_message_timestamp
 from gajim.common.modules.message_util import get_occupant_info
-from gajim.common.modules.message_util import get_open_graph_data
+from gajim.common.modules.message_util import get_open_graph
 from gajim.common.modules.message_util import get_reply
 from gajim.common.modules.message_util import get_security_label
 from gajim.common.modules.misc import parse_oob
@@ -177,6 +177,14 @@ class Message(BaseModule):
                                         stanza_id=stanza_id))
                 return
 
+        resource = jid.resource
+        if m_type == MessageType.PM and direction == ChatDirection.OUTGOING:
+            # Outgoing PMs which we receive as carbons or via MAM
+            # have our bound jid in the from attribute, not the joined MUC
+            # resource. This means for these kind of messages the nickname
+            # cannot be determined
+            resource = None
+
         occupant = None
         if (m_type in (MessageType.GROUPCHAT, MessageType.PM)
                 and not jid.is_bare):
@@ -190,6 +198,7 @@ class Message(BaseModule):
                 remote_jid,
                 self._get_own_bare_jid(),
                 direction,
+                m_type,
                 timestamp,
                 contact,
                 properties
@@ -219,7 +228,7 @@ class Message(BaseModule):
         if properties.correction is not None:
             correction_id = properties.correction.id
 
-        og_data = get_open_graph_data(properties.open_graph)
+        og_data = get_open_graph(properties.open_graph)
 
         message_data = mod.Message(
             account_=self._account,
@@ -228,7 +237,7 @@ class Message(BaseModule):
             direction=direction,
             timestamp=timestamp,
             state=MessageState.ACKNOWLEDGED,
-            resource=jid.resource,
+            resource=resource,
             text=message_text,
             id=message_id,
             stanza_id=stanza_id,
@@ -370,7 +379,14 @@ class Message(BaseModule):
 
             resource = muc_data.nick
             real_jid = self._client.get_own_jid().new_as_bare()
-            occupant_id = muc_data.occupant_id or real_jid
+            if message.type == MessageType.PM:
+                # Use real jid as occupant-id because sent MUC PMs which we
+                # receive via carbons or MAM do not have the occupant-id
+                # which would lead to outgoing message being stored with
+                # different occupant-ids
+                occupant_id = real_jid
+            else:
+                occupant_id = muc_data.occupant_id or real_jid
 
             occupant = mod.Occupant(
                 account_=self._account,
@@ -454,6 +470,10 @@ class Message(BaseModule):
         if message.oob_url is not None:
             oob_data.append(mod.OOB(url=message.oob_url, description=None))
 
+        open_graph_data: list[mod.OpenGraph] = []
+        if message.open_graph_data:
+            open_graph_data = get_open_graph(message.open_graph_data)
+
         message_data = mod.Message(
             account_=self._account,
             remote_jid_=remote_jid,
@@ -470,6 +490,7 @@ class Message(BaseModule):
             correction_id=message.correct_id,
             encryption_=encryption_data,
             oob=oob_data,
+            og=open_graph_data,
             reply=reply,
             security_label_=securitylabel_data,
             occupant_=occupant,
@@ -531,6 +552,12 @@ def build_message_stanza(message: OutgoingMessage, own_jid: JID) -> nbxmpp.Messa
                         message.reply_data.fallback_start,
                         message.reply_data.fallback_end)
 
+    # OGP link previews
+    if message.open_graph_data is not None:
+        for about_url, open_graph_data in message.open_graph_data.items():
+            stanza.addOpenGraph(about_url, open_graph_data.to_nbxmpp())
+
+    # XEP-0258
     if message.sec_label:
         stanza.addChild(node=message.sec_label.to_node())
 
