@@ -292,7 +292,6 @@ StringAccountSettings = Literal[
     'account_color',
     'account_label',
     'address',
-    'attached_gpg_keys',
     'avatar_sha',
     'client_cert',
     'custom_host',
@@ -302,8 +301,6 @@ StringAccountSettings = Literal[
     'filetransfer_preference',
     'gc_send_chatstate_default',
     'http_auth',
-    'keyid',
-    'keyname',
     'last_status',
     'last_status_msg',
     'password',
@@ -380,7 +377,6 @@ ACCOUNT_SETTINGS = {
         'address': '',
         'anonymous_auth': False,
         'answer_receipts': True,
-        'attached_gpg_keys': '',
         'autoauth': False,
         'autoconnect': True,
         'autojoin_sync': True,
@@ -404,8 +400,6 @@ ACCOUNT_SETTINGS = {
         'gc_send_marker_public_default': False,
         'http_auth': 'ask',
         'ignore_unknown_contacts': False,
-        'keyid': '',
-        'keyname': '',
         'last_status': 'online',
         'last_status_msg': '',
         'password': '',
@@ -500,33 +494,6 @@ PLUGIN_SETTINGS = {
 }
 
 
-STATUS_PRESET_SETTINGS = {
-    'message': '',
-}
-
-
-STATUS_PRESET_EXAMPLES = {
-    _('Sleeping'): {
-        'message': _('ZZZZzzzzzZZZZZ')
-    },
-    _('Back soon'): {
-        'message': _('Back in some minutes.')
-    },
-    _('Eating'): {
-        'message': _('I’m eating.')
-    },
-    _('Movie'): {
-        'message': _('I’m watching a movie.')
-    },
-    _('Working'): {
-        'message': _('I’m working.')
-    },
-    _('Out'): {
-        'message': _('I’m out enjoying life.')
-    }
-}
-
-
 PROXY_SETTINGS = {
     'type': 'socks5',
     'host': '',
