@@ -178,10 +178,13 @@ class ContactInfo(GajimAppWindow, EventHelper):
 
         self._save_annotation()
 
-        for device_grid in self._devices.values():
-            self._ui.devices_page.remove(device_grid)
+        for device_info in self._devices.values():
+            device_info.run_destroy()
+            self._ui.devices_page.remove(device_info)
+
         self._devices.clear()
 
+        self._switcher.run_destroy()
         del self._switcher
 
         self._client.disconnect_all_from_obj(self)
@@ -283,7 +286,9 @@ class ContactInfo(GajimAppWindow, EventHelper):
             # All settings for PM are derived from the groupchat settings
             # so we dont need to show them
             contact_settings = ContactSettings(self.account, self._bare_contact.jid)
-            self._ui.contact_settings_box.add(contact_settings)
+            self._ui.settings_box.insert_child_after(
+                contact_settings, self._ui.subscription_box
+            )
 
         params = AccountJidParam(account=self.account, jid=self.contact.jid)
         self._ui.remove_history_button.set_action_target_value(params.to_variant())
@@ -548,9 +553,8 @@ class DeviceInfo(Adw.PreferencesGroup):
 
         self._waiting_for_info = 2
 
-    def do_unroot(self) -> None:
-        Adw.PreferencesGroup.do_unroot(self)
-        del self._contact
+    def run_destroy(self) -> None:
+        app.check_finalize(self)
 
     def set_entity_time(self, entity_time: str | None) -> None:
         if entity_time is not None:
