@@ -24,7 +24,6 @@ from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import can_add_to_roster
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
-from gajim.common.preview import Preview
 from gajim.common.storage.archive.const import ChatDirection
 from gajim.common.storage.archive.models import Message
 from gajim.common.structs import VariantMixin
@@ -80,7 +79,6 @@ class GajimMenu(Gio.Menu):
 def make_menu_item(
     label: str, action: str | None = None, value: MenuValueT = None
 ) -> Gio.MenuItem:
-
     item = Gio.MenuItem.new(label)
 
     if value is None:
@@ -215,7 +213,6 @@ def get_groupchat_menu(contact: GroupchatContact) -> GajimMenu:
 
 
 def get_account_menu(account: str) -> GajimMenu:
-
     client = app.get_client(account)
     server_jid = client.get_own_jid().domain
     assert server_jid is not None
@@ -223,29 +220,15 @@ def get_account_menu(account: str) -> GajimMenu:
     params = GLib.Variant("(sas)", (account, [server_jid]))
 
     menuitems: MenuItemListT = [
-        (_("Profile"), f"app.{account}-profile", account),
         (_("Discover Services…"), f"app.{account}-services", account),
         (_("Execute Command…"), f"app.{account}-execute-command", params),
-        (_("Server Info"), f"app.{account}-server-info", account),
-    ]
-
-    menu = GajimMenu.from_list(menuitems)
-
-    advanced_menuitems: MenuItemListT = [
-        (_("Manage Contact List"), f"app.{account}-manage-roster", account),
-        (_("Archiving Preferences"), f"app.{account}-archive", account),
-        (_("Blocking List"), f"app.{account}-blocking", account),
         (_("PEP Configuration"), f"app.{account}-pep-config", account),
-        (_("Synchronize History…"), f"app.{account}-sync-history", account),
     ]
 
-    menu.append_submenu(_("Advanced"), GajimMenu.from_list(advanced_menuitems))
-
-    return menu
+    return GajimMenu.from_list(menuitems)
 
 
 def get_encryption_menu() -> GajimMenu:
-
     menuitems: MenuItemListT = [
         (_("Disabled"), "win.set-encryption", ""),
         ("OMEMO", "win.set-encryption", "OMEMO"),
@@ -259,8 +242,8 @@ def get_encryption_menu() -> GajimMenu:
 def get_message_input_extra_context_menu() -> Gio.Menu:
     menuitems: MenuItemListT = [
         (_("Clear"), "win.input-clear", None),
-        (_("Paste as Code Block"), "win.paste-as-code-block", None),
-        (_("Paste as Quote"), "win.paste-as-quote", None),
+        (_("Paste as Code Block"), "win.input-paste-as-code-block", None),
+        (_("Paste as Quote"), "win.input-paste-as-quote", None),
     ]
 
     menu = GajimMenu.from_list(menuitems)
@@ -278,7 +261,7 @@ def get_conv_action_context_menu(account: str, selected_text: str) -> Gio.Menu:
 
     # Wikipedia search
     if app.settings.get("always_english_wikipedia"):
-        uri = f"https://en.wikipedia.org/wiki/" f"Special:Search?search={uri_text}"
+        uri = f"https://en.wikipedia.org/wiki/Special:Search?search={uri_text}"
     else:
         uri = (
             f"https://{get_short_lang_code()}.wikipedia.org/"
@@ -295,7 +278,7 @@ def get_conv_action_context_menu(account: str, selected_text: str) -> Gio.Menu:
     if dict_link == "WIKTIONARY":
         # Default is wikitionary.org
         if app.settings.get("always_english_wiktionary"):
-            uri = f"https://en.wiktionary.org/wiki/" f"Special:Search?search={uri_text}"
+            uri = f"https://en.wiktionary.org/wiki/Special:Search?search={uri_text}"
         else:
             uri = (
                 f"https://{get_short_lang_code()}.wiktionary.org/"
@@ -433,7 +416,6 @@ def get_subscription_menu(account: str, jid: JID) -> GajimMenu:
 
 
 def get_start_chat_button_menu() -> GajimMenu:
-
     value = GLib.Variant("as", ["", ""])
 
     menuitems: MenuItemListT = [
@@ -471,7 +453,6 @@ def get_start_chat_row_menu(account: str, jid: JID | None) -> GajimMenu | None:
 def get_chat_list_row_menu(
     workspace_id: str, account: str, jid: JID, pinned: bool
 ) -> GajimMenu:
-
     client = app.get_client(account)
     contact = client.get_module("Contacts").get_contact(jid)
     assert isinstance(contact, BareContact | GroupchatContact | GroupchatParticipant)
@@ -535,7 +516,6 @@ def get_chat_list_row_menu(
 def get_workspace_params(
     current_workspace_id: str, account: str, jid: JID
 ) -> Iterator[tuple[str, ChatListEntryParam]]:
-
     for workspace_id in app.settings.get_workspaces():
         if workspace_id == current_workspace_id:
             continue
@@ -552,7 +532,6 @@ def get_workspace_params(
 def get_groupchat_admin_menu(
     self_contact: types.GroupchatParticipant, contact: types.GroupchatParticipant
 ) -> GajimMenu:
-
     menu = GajimMenu()
 
     if contact.real_jid is None:
@@ -590,7 +569,6 @@ def get_groupchat_admin_menu(
 def get_groupchat_mod_menu(
     self_contact: types.GroupchatParticipant, contact: types.GroupchatParticipant
 ) -> GajimMenu:
-
     menu = GajimMenu()
 
     if not contact.is_available:
@@ -623,13 +601,15 @@ def get_groupchat_participant_menu(
     self_contact: types.GroupchatParticipant,
     contact: types.GroupchatParticipant,
 ) -> GajimMenu:
-
     group_chat = self_contact.room
     disco = group_chat.get_disco()
     assert disco is not None
     muc_prefer_direct_msg = app.settings.get("muc_prefer_direct_msg")
-    if disco.muc_is_nonanonymous and muc_prefer_direct_msg:
-        assert contact.real_jid is not None
+    if (
+        disco.muc_is_nonanonymous
+        and muc_prefer_direct_msg
+        and contact.real_jid is not None
+    ):
         dm_params = AddChatActionParams(
             account=account, jid=contact.real_jid, type="chat", select=True
         )
@@ -689,7 +669,6 @@ def get_chat_row_menu(
     message: Message,
     original_message: Message,
 ) -> GajimMenu:
-
     menu_items: MenuItemListT = []
     is_retracted = original_message.is_retracted()
 
@@ -703,7 +682,7 @@ def get_chat_row_menu(
 
     if show_correction and not is_retracted:
         menu_items.append(
-            (p_("Message row action", "Correct…"), "win.correct-message", None)
+            (p_("Message row action", "Edit…"), "win.correct-message", None)
         )
 
     param = None
@@ -755,7 +734,6 @@ def _get_retract_param(
     contact: types.ChatContactT,
     original_message: Message,
 ) -> RetractMessageParam | None:
-
     if original_message.direction == ChatDirection.INCOMING:
         return
 
@@ -823,39 +801,18 @@ def _get_moderate_params(
     return single_param, multiple_param
 
 
-def get_preview_menu(preview: Preview) -> GajimMenu:
-    menu_items: MenuItemListT = []
+def get_preview_menu(uri: str, *, encrypted: bool = False) -> GajimMenu:
+    menu_items: MenuItemListT = [
+        (_("_Copy Link"), "app.copy-text", uri),
+    ]
 
-    download = (_("_Download"), "win.preview-download", preview.id)
-    open_file = (_("_Open"), "win.preview-open", preview.id)
-    save_as = (_("_Save as…"), "win.preview-save-as", preview.id)
-    open_folder = (_("Open _Folder"), "win.preview-open-folder", preview.id)
-    copy_link = (_("_Copy Link"), "win.preview-copy-link", preview.id)
-    open_link = (_("Open Link in _Browser"), "win.preview-open-link", preview.id)
-
-    if preview.is_geo_uri:
-        menu_items.append(open_file)
-        menu_items.append(copy_link)
-        return GajimMenu.from_list(menu_items)
-
-    if preview.orig_exists:
-        menu_items.append(open_file)
-        menu_items.append(save_as)
-        menu_items.append(open_folder)
-    else:
-        if not preview.download_in_progress:
-            menu_items.append(download)
-
-    menu_items.append(copy_link)
-
-    if not preview.is_aes_encrypted:
-        menu_items.append(open_link)
+    if not encrypted:
+        menu_items.append((_("Open Link in _Browser"), "app.open-link", uri))
 
     return GajimMenu.from_list(menu_items)
 
 
 def get_format_menu() -> GajimMenu:
-
     menuitems: MenuItemListT = [
         (_("bold"), "win.input-bold", None),
         (_("italic"), "win.input-italic", None),
@@ -879,40 +836,52 @@ def get_workspace_menu(workspace_id: str) -> GajimMenu:
     return GajimMenu.from_list(menuitems)
 
 
-def get_manage_roster_menu(groups: list[str], single_selection: bool) -> GajimMenu:
+def get_manage_roster_menu(
+    account: str, groups: list[str], single_selection: bool
+) -> GajimMenu:
     menu = GajimMenu()
 
-    menuitems: MenuItemListT = [(_("New Group…"), "win.move-to-new-group", None)]
+    menuitems: MenuItemListT = [
+        (_("New Group…"), f"win.{account}-move-to-new-group", None)
+    ]
 
     for group in groups:
-        menuitems.append((group, "win.move-to-group", group))
+        menuitems.append((group, f"win.{account}-move-to-group", group))
 
     menu.append_submenu(_("Move to Group"), GajimMenu.from_list(menuitems))
 
-    menuitems: MenuItemListT = [(_("New Group…"), "win.add-to-new-group", None)]
+    menuitems: MenuItemListT = [
+        (_("New Group…"), f"win.{account}-add-to-new-group", None)
+    ]
 
     for group in groups:
-        menuitems.append((group, "win.add-to-group", group))
+        menuitems.append((group, f"win.{account}-add-to-group", group))
 
     menu.append_submenu(_("Add to Group"), GajimMenu.from_list(menuitems))
 
-    menu.add_item(_("Remove from Group"), "win.remove-from-group", None)
+    menu.add_item(_("Remove from Group"), f"win.{account}-remove-from-group", None)
     if single_selection:
-        menu.add_item(_("Rename…"), "win.change-name", None)
+        menu.add_item(_("Rename…"), f"win.{account}-change-name", None)
 
-    menu.add_item(_("Remove from Contact List…"), "win.remove-from-roster", None)
+    menu.add_item(
+        _("Remove from Contact List…"), f"win.{account}-remove-from-roster", None
+    )
 
     return menu
 
 
-def get_manage_roster_import_menu(accounts: list[tuple[str, str]]) -> GajimMenu:
+def get_manage_roster_import_menu(
+    account: str, accounts: list[tuple[str, str]]
+) -> GajimMenu:
     menu = GajimMenu()
-    menu.add_item(_("Import from File"), "win.import-from-file", None)
+    menu.add_item(_("Import from File"), f"win.{account}-import-from-file", None)
 
     if accounts:
         menuitems: MenuItemListT = []
-        for account, label in accounts:
-            menuitems.append((label, "win.import-from-account", account))
+        for remote_account, label in accounts:
+            menuitems.append(
+                (label, f"win.{account}-import-from-account", remote_account)
+            )
 
         menu.append_submenu(_("Import from Account"), GajimMenu.from_list(menuitems))
     else:
