@@ -36,6 +36,7 @@ from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.open_graph_parser import OpenGraphData
 from gajim.common.storage.archive import models as mod
 from gajim.common.structs import ReplyData
 from gajim.common.types import ChatContactT
@@ -47,6 +48,7 @@ from gajim.gtk.emoji_chooser import EmojiChooser
 from gajim.gtk.menus import get_encryption_menu
 from gajim.gtk.menus import get_format_menu
 from gajim.gtk.message_input import MessageInputTextView
+from gajim.gtk.message_url_previews import MessageURLPreviews
 from gajim.gtk.referenced_message import ReplyBox
 from gajim.gtk.security_label_selector import SecurityLabelSelector
 from gajim.gtk.util.classes import SignalManager
@@ -103,23 +105,26 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             self._security_label_selector, self._ui.input_wrapper
         )
 
-        self.msg_textview = MessageInputTextView(self)
-        self._connect(self.msg_textview, "buffer-changed", self._on_buffer_changed)
-        self._connect(self.msg_textview, "paste-clipboard", self._on_paste_clipboard)
+        self._message_url_previews = MessageURLPreviews()
+        self._ui.link_previews_box.append(self._message_url_previews)
 
-        self._ui.box.append(self.msg_textview.get_completion_popover())
+        self._message_input = MessageInputTextView(self)
+        self._connect(self._message_input, "buffer-changed", self._on_buffer_changed)
+        self._connect(self._message_input, "paste-clipboard", self._on_paste_clipboard)
+
+        self._ui.box.append(self._message_input.get_completion_popover())
 
         controller = Gtk.EventControllerKey()
-        self._connect(controller, "key-pressed", self._on_msg_textview_key_pressed)
-        self.msg_textview.add_controller(controller)
+        self._connect(controller, "key-pressed", self._on_message_input_key_pressed)
+        self._message_input.add_controller(controller)
 
-        self._ui.input_scrolled.set_child(self.msg_textview)
+        self._ui.input_scrolled.set_child(self._message_input)
 
         self._wait_queue_resize = None
         self._vscrollbar_min_height = 0
         vadjustment = self._ui.input_scrolled.get_vadjustment()
         self._connect(vadjustment, "changed", self._on_input_scrolled_changed)
-        self._connect(self.msg_textview, "realize", self._on_view_realize)
+        self._connect(self._message_input, "realize", self._on_view_realize)
 
         self._ui.sendfile_button.set_tooltip_text(_("No File Transfer available"))
         self._ui.formattings_button.set_menu_model(get_format_menu())
@@ -149,6 +154,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             [
                 ("message-sent", ged.GUI2, self._on_message_sent),
                 ("register-actions", ged.GUI2, self._on_register_actions),
+                ("muc-disco-update", ged.GUI2, self._on_muc_disco_update),
             ]
         )
 
@@ -166,15 +172,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
     def _on_register_actions(self, _event: events.RegisterActions) -> None:
         actions = [
-            "input-bold",
-            "input-italic",
-            "input-strike",
-            "input-clear",
-            "show-emoji-chooser",
-            "paste-as-quote",
-            "paste-as-code-block",
             "quote",
-            "mention",
             "reply",
             "correct-message",
         ]
@@ -192,6 +190,12 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         action = app.window.get_action("send-file-httpupload")
         self._connect(action, "notify::enabled", self._on_send_file_enabled_changed)
 
+    def _on_muc_disco_update(self, event: events.MucDiscoUpdate) -> None:
+        if self._contact is None or event.jid != self._contact.jid:
+            return
+
+        self._update_encryption_state()
+
     def get_current_contact(self) -> ChatContactT:
         assert self._contact is not None
         return self._contact
@@ -199,9 +203,12 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
     def get_seclabel(self) -> SecurityLabel | None:
         return self._security_label_selector.get_seclabel()
 
+    def get_message_input(self) -> MessageInputTextView:
+        return self._message_input
+
     def _on_input_scrolled_changed(self, adj: Gtk.Adjustment) -> None:
         scrolled = self._ui.input_scrolled
-        view = self.msg_textview
+        view = self._message_input
 
         """
         Fixes #12158. The textview would only show the last row(s),
@@ -229,22 +236,13 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             > scrolled.get_max_content_height() - 2 * self._vscrollbar_min_height
         )
 
-        self._start_queue_resize_if_needed()
-
-    def _start_queue_resize_if_needed(self) -> None:
-        if self._wait_queue_resize is None:
-            self._wait_queue_resize = GLib.timeout_add(50, self._queue_resize_if_needed)
-
-    def _queue_resize_if_needed(self) -> bool:
-        if (
-            self._ui.input_scrolled.get_height()
-            == self._ui.input_scrolled.get_size_request()[1]
-        ):
-            self._wait_queue_resize = None
-            return GLib.SOURCE_REMOVE
+        if self._contact is not None and self._wait_queue_resize is None:
+            self._wait_queue_resize = GLib.timeout_add(100, self._queue_resize)
 
+    def _queue_resize(self) -> bool:
+        self._wait_queue_resize = None
         self._ui.input_scrolled.queue_resize()
-        return GLib.SOURCE_CONTINUE
+        return GLib.SOURCE_REMOVE
 
     def _on_view_realize(self, _view: Gtk.TextView) -> None:
         min_size, _ = self._ui.input_scrolled.get_preferred_size()
@@ -256,40 +254,19 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         emoji_chooser.set_emoji_picked_func(self._on_emoji_picked)
 
     def _on_emoji_picked(self, _emoji_chooser: EmojiChooser, text: str) -> None:
-        self.msg_textview.insert_text(text)
-        GLib.idle_add(self.msg_textview.grab_focus_delayed)
+        self._message_input.insert_text(text)
+        GLib.idle_add(self._message_input.grab_focus_delayed)
 
     def _on_action(
         self, action: Gio.SimpleAction, param: GLib.Variant | None
     ) -> int | None:
-
         if self._contact is None:
             return
 
         action_name = action.get_name()
         log.info("Activate action: %s", action_name)
 
-        if action_name == "input-clear":
-            self._on_clear()
-
-        elif action_name.startswith("input-"):
-            self._on_format(action_name)
-
-        elif action_name == "show-emoji-chooser":
-            self.msg_textview.emit("insert-emoji")
-            self._ui.emoticons_button.set_active(False)
-
-        elif action_name == "quote":
-            assert param
-            self.msg_textview.insert_as_quote(param.get_string())
-
-        elif action_name == "paste-as-quote":
-            self.msg_textview.paste_as_quote()
-
-        elif action_name == "paste-as-code-block":
-            self.msg_textview.paste_as_code_block()
-
-        elif action_name == "reply":
+        if action_name == "reply":
             assert param
             pk = param.get_uint32()
             original_message = app.storage.archive.get_message_with_pk(pk)
@@ -297,10 +274,6 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
                 return
             self._enable_reply_mode(original_message)
 
-        elif action_name == "mention":
-            assert param
-            self.msg_textview.mention_participant(param.get_string())
-
         elif action_name == "correct-message":
             self.toggle_message_correction()
 
@@ -337,24 +310,13 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
                 }
             )
 
-        encryption = contact.settings.get("encryption")
-        encryption_available = self._is_encryption_available(contact)
-        if not encryption_available and encryption:
-            # Disable encryption if chat was encrypted before, but due to
-            # changed circumstances, encryption is not applicable anymore
-            # (i.e. group chat configuration changed).
-            contact.settings.set("encryption", "")
-            encryption = ""
-
-        action = app.window.get_action("set-encryption")
-        action.set_state(GLib.Variant("s", encryption))
-        self._update_encryption_button(encryption, is_available=encryption_available)
-        self._update_encryption_details_button(encryption)
+        self._update_encryption_state()
         self._update_send_file_button_tooltip()
 
         self._set_chatstate(True)
 
-        self.msg_textview.switch_contact(contact)
+        self._message_url_previews.switch_contact(contact)
+        self._message_input.switch_contact(contact)
 
         self._chat_state_indicator.switch_contact(contact)
 
@@ -369,7 +331,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         if self._contact is None:
             return
 
-        text = self.msg_textview.get_text()
+        text = self._message_input.get_text()
 
         reply_pk = None
         reply_data = self._reply_box.get_message_reply()
@@ -377,7 +339,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             reply_pk = reply_data.pk
 
         draft = None
-        if text or reply_pk is not None:
+        if any([text, reply_pk]):
             draft = Draft(text, reply_pk)
 
         app.storage.drafts.set(self._contact, draft)
@@ -388,7 +350,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         if draft is None:
             return
 
-        self.msg_textview.insert_text(draft.text)
+        self._message_input.insert_text(draft.text)
 
         if draft.reply_pk is None:
             return
@@ -418,7 +380,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         self._client = None
 
     def get_text(self) -> str:
-        return self.msg_textview.get_text()
+        return self._message_input.get_text()
 
     @property
     def _is_correcting(self) -> bool:
@@ -439,8 +401,8 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             return
 
         if clear:
-            self.msg_textview.clear()
-        self.msg_textview.insert_as_quote(text)
+            self._message_input.clear()
+        self._message_input.insert_as_quote(text)
 
     def process_escape(self) -> bool:
         if not self._is_correcting and not self._reply_box.is_in_reply_mode:
@@ -454,7 +416,8 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         self._disable_reply_mode()
 
     def reset_state_after_send(self) -> None:
-        self.msg_textview.clear()
+        self._message_input.clear()
+        self._message_url_previews.clear()
         self._cancel_action()
         assert self._contact is not None
         app.storage.drafts.set(self._contact, None)
@@ -464,7 +427,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
         if self._is_correcting:
             self._set_correcting(None)
-            self.msg_textview.end_correction()
+            self._message_input.end_correction()
             self._ui.edit_box.set_visible(False)
             return
 
@@ -486,7 +449,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
         message = message.get_last_correction() or message
 
-        self.msg_textview.start_correction(message)
+        self._message_input.start_correction(message)
         self._ui.edit_box.set_visible(True)
 
         referenced_message = message.get_referenced_message()
@@ -565,7 +528,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
     def _set_chatstate(self, state: bool) -> None:
         assert self._client is not None
         if state:
-            if self.msg_textview.has_text:
+            if self._message_input.has_text:
                 self._client.get_module("Chatstate").set_chatstate(
                     self._contact, Chatstate.PAUSED
                 )
@@ -579,7 +542,6 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             )
 
     def _change_encryption(self, action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         new_state = param.get_string()
         action_state = action.get_state()
         assert action_state is not None
@@ -603,8 +565,23 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         self._update_encryption_button(new_state, is_available=True)
         self._update_encryption_details_button(new_state)
 
-    def _update_encryption_button(self, encryption: str, *, is_available: bool) -> None:
+    def _update_encryption_state(self) -> None:
+        contact = self.get_current_contact()
+        encryption = contact.settings.get("encryption")
+        encryption_available = self._is_encryption_available(contact)
+        if not encryption_available and encryption:
+            # Disable encryption if chat was encrypted before, but due to
+            # changed circumstances, encryption is not applicable anymore
+            # (i.e. group chat configuration changed).
+            contact.settings.set("encryption", "")
+            encryption = ""
 
+        action = app.window.get_action("set-encryption")
+        action.set_state(GLib.Variant("s", encryption))
+        self._update_encryption_button(encryption, is_available=encryption_available)
+        self._update_encryption_details_button(encryption)
+
+    def _update_encryption_button(self, encryption: str, *, is_available: bool) -> None:
         contact = self.get_current_contact()
 
         if is_available:
@@ -703,17 +680,9 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             f"encryption_dialog{encryption}", app.window.get_control()
         )
 
-    def _on_format(self, name: str) -> None:
-        name = name.removeprefix("input-")
-        self.msg_textview.apply_formatting(name)
-
-    def _on_clear(self) -> None:
-        self.msg_textview.clear()
-
     def _on_send_file_enabled_changed(
         self, action: Gio.SimpleAction, _param: GObject.ParamSpec
     ) -> None:
-
         self._update_send_file_button_tooltip()
 
     def _update_send_file_button_tooltip(self):
@@ -748,11 +717,13 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         if self._contact is None:
             return
 
-        has_text = self.msg_textview.has_text
+        has_text = self._message_input.has_text
         app.window.get_action("send-message").set_enabled(
             allow_send_message(has_text, self._contact)
         )
 
+        self._message_url_previews.generate_url_previews(self._message_input.get_text())
+
         encryption_name = self._contact.settings.get("encryption")
 
         if has_text and encryption_name:
@@ -768,7 +739,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
         client.get_module("Chatstate").set_chatstate(self._contact, Chatstate.COMPOSING)
 
-    def _on_msg_textview_key_pressed(
+    def _on_message_input_key_pressed(
         self,
         _event_controller_key: Gtk.EventControllerKey,
         keyval: int,
@@ -788,11 +759,11 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
                 return Gdk.EVENT_STOP
 
             if keyval == Gdk.KEY_z:
-                self.msg_textview.undo()
+                self._message_input.undo()
                 return Gdk.EVENT_STOP
 
             if keyval == Gdk.KEY_y:
-                self.msg_textview.redo()
+                self._message_input.redo()
                 return Gdk.EVENT_STOP
 
             if keyval == Gdk.KEY_Up:
@@ -801,24 +772,24 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
         if keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter):
             if state & Gdk.ModifierType.SHIFT_MASK:
-                self.msg_textview.insert_newline()
+                self._message_input.insert_newline()
                 return Gdk.EVENT_STOP
 
             if state & Gdk.ModifierType.CONTROL_MASK:
                 if not app.settings.get("send_on_ctrl_enter"):
-                    self.msg_textview.insert_newline()
+                    self._message_input.insert_newline()
                     return Gdk.EVENT_STOP
             else:
                 if app.settings.get("send_on_ctrl_enter"):
-                    self.msg_textview.insert_newline()
+                    self._message_input.insert_newline()
                     return Gdk.EVENT_STOP
 
             assert self._contact is not None
 
             # Reset IMContext to clear preedit state
-            self.msg_textview.reset_im_context()
+            self._message_input.reset_im_context()
 
-            message = self.msg_textview.get_text()
+            message = self._message_input.get_text()
 
             try:
                 handled = app.commands.parse(self._contact.type_string, message)
@@ -826,13 +797,13 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
                 return Gdk.EVENT_STOP
 
             if handled:
-                self.msg_textview.clear()
+                self._message_input.clear()
                 return Gdk.EVENT_STOP
 
             if not app.account_is_available(self._contact.account):
                 # we are not connected
                 InformationAlertDialog(
-                    _("Not Connected"),
+                    _("You are offline"),
                     _("Your message can not be sent until you are connected."),
                 )
                 return Gdk.EVENT_STOP
@@ -849,7 +820,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
     def _enable_reply_mode(self, original_message: mod.Message) -> None:
         assert self._contact is not None
         self._reply_box.enable_reply_mode(self._contact, original_message)
-        self.msg_textview.grab_focus()
+        self._message_input.grab_focus()
 
     def _disable_reply_mode(self, *args: Any) -> None:
         self._reply_box.disable_reply_mode()
@@ -861,11 +832,20 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
         return message_reply
 
+    def get_open_graph_data(self) -> dict[str, OpenGraphData] | None:
+        return self._message_url_previews.get_open_graph_data()
+
     def _on_paste_clipboard(self, textview: MessageInputTextView) -> None:
         clipboard = self.get_clipboard()
         formats = clipboard.get_formats()
         mime_types = formats.get_mime_types()
 
+        log.debug(
+            "Clipboard paste received, mime-types: %s, gtypes: %s",
+            mime_types,
+            formats.get_gtypes(),
+        )
+
         if mime_types is None:
             log.warning("Could not determine clipboard mime types")
             return
