@@ -7,6 +7,7 @@ from typing import cast
 
 import hashlib
 import logging
+import sys
 from pathlib import Path
 from urllib.parse import urlparse
 
@@ -120,9 +121,6 @@ class PreviewWidget(Gtk.Box, SignalManager):
         self._http_obj = None
         self._state = PreviewState.INIT
 
-        self._drag_hotspot_x: float = 0
-        self._drag_hotspot_y: float = 0
-
         drag_source = Gtk.DragSource(actions=Gdk.DragAction.COPY)
         self._connect(drag_source, "prepare", self._on_drag_prepare)
         self._connect(drag_source, "drag-begin", self._on_drag_begin)
@@ -197,7 +195,6 @@ class PreviewWidget(Gtk.Box, SignalManager):
         del self._menu_popover
         del self._http_obj
         Gtk.Box.do_unroot(self)
-        app.check_finalize(self)
 
     def get_text(self) -> str:
         return self._uri
@@ -211,9 +208,6 @@ class PreviewWidget(Gtk.Box, SignalManager):
         if self._state not in (PreviewState.DOWNLOADED, PreviewState.DISPLAY):
             return None
 
-        self._drag_hotspot_x = x
-        self._drag_hotspot_y = y
-
         value = GObject.Value()
         value.init(PreviewWidget)
         value.set_object(self)
@@ -224,8 +218,8 @@ class PreviewWidget(Gtk.Box, SignalManager):
         Gtk.DragIcon.set_from_paintable(
             drag,
             Gtk.WidgetPaintable().new(self),
-            hot_x=int(self._drag_hotspot_x),
-            hot_y=int(self._drag_hotspot_y),
+            hot_x=0,
+            hot_y=0,
         )
 
         app.window.highlight_dnd_targets(self, True)
@@ -259,13 +253,15 @@ class PreviewWidget(Gtk.Box, SignalManager):
             widget = None
             if is_image(self._mime_type) or is_video(self._mime_type):
                 assert self._mime_type is not None
-                widget = ImagePreviewWidget(
-                    self._filename,
-                    self._file_size,
-                    self._mime_type,
-                    self._orig_path,
-                    self._thumb_path,
-                )
+                if sys.platform != "darwin":
+                    # https://dev.gajim.org/gajim/gajim/-/issues/12625
+                    widget = ImagePreviewWidget(
+                        self._filename,
+                        self._file_size,
+                        self._mime_type,
+                        self._orig_path,
+                        self._thumb_path,
+                    )
 
             elif is_audio(self._mime_type):
                 if (
