@@ -6,13 +6,19 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import cast
 from typing import Literal
 
 import datetime
+import gc
 import logging
+import os
+import pprint
 import sys
+import weakref
 import xml.etree.ElementTree as ET
 from collections.abc import Iterator
+from functools import partial
 from functools import wraps
 from pathlib import Path
 
@@ -21,9 +27,10 @@ from gi.repository import Adw
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
+from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Pango
-from packaging.version import Version as V
+from packaging.requirements import Requirement
 
 from gajim.common import app
 from gajim.common import types
@@ -59,10 +66,6 @@ def get_gtk_version() -> str:
     )
 
 
-def has_min_gtk_version(required_version: str) -> bool:
-    return V(required_version) >= V(get_gtk_version())
-
-
 def get_adw_version() -> str:
     return "%i.%i.%i" % (
         Adw.get_major_version(),
@@ -71,6 +74,19 @@ def get_adw_version() -> str:
     )
 
 
+def gi_gui_package_version(requirement: str):
+    req = Requirement(requirement)
+    match req.name:
+        case "Gtk":
+            version = get_gtk_version()
+        case "Adw":
+            version = get_adw_version()
+        case _:
+            raise ValueError("Unknown lib name: %s" % req.name)
+
+    return req.specifier.contains(version)
+
+
 def scroll_to(widget: Gtk.ScrolledWindow, pos: Literal["top", "bottom"]) -> bool:
     """Scrolls to `pos` of a GtkScrolledWindow.
 
@@ -140,9 +156,16 @@ def get_contact_name_for_message(
         return resource
 
     elif db_row.type == MessageType.PM:
-        resource = db_row.resource
-        assert resource is not None
-        return resource
+        if occupant := db_row.occupant:
+            if name := occupant.nickname:
+                return name
+
+        if db_row.resource is None:
+            if db_row.direction == ChatDirection.OUTGOING:
+                return _("Me")
+            # This should never happen better degrade gracefully
+            return "Unknown"
+        return db_row.resource
 
     else:
         raise ValueError
@@ -168,8 +191,10 @@ def get_avatar_for_message(
             )
 
         case MessageType.CHAT:
-            assert isinstance(contact, BareContact)
-            return contact.get_avatar(size, scale, add_show=False)
+            if isinstance(contact, BareContact):
+                return contact.get_avatar(size, scale, add_show=False)
+            log.warning("Unexpected contact type %r, unable to get avatar", contact)
+            return None
 
         case _:
             raise ValueError(f"Unhandled type: {db_row.type}")
@@ -289,6 +314,27 @@ def iterate_children(widget: Gtk.Widget) -> Iterator[Gtk.Widget]:
         yield child
 
 
+def iterate_widget_tree(
+    widget: Gtk.Widget, parent: str | None = None, only_children: bool = False
+) -> Iterator[tuple[Gtk.Widget, str | None]]:
+    if not only_children:
+        yield widget, parent
+
+    if parent is None:
+        parent = widget.__class__.__name__
+    else:
+        parent += f" -> {widget.__class__.__name__}"
+
+    child = widget.get_first_child()
+    if child is None:
+        return
+
+    yield from iterate_widget_tree(child, parent)
+
+    while child := child.get_next_sibling():
+        yield from iterate_widget_tree(child, parent)
+
+
 def convert_surface_to_texture(surface: cairo.ImageSurface) -> Gdk.Texture:
     memoryv = surface.get_data()
     assert memoryv is not None
@@ -402,3 +448,120 @@ def remove_css_class(widget: Gtk.Widget, css_class_name: str) -> None:
     for css_class in list(widget.get_css_classes()):
         if css_class_name in css_class:
             widget.remove_css_class(css_class)
+
+
+class ObjTracker:
+    def __init__(self) -> None:
+        self._gobjects: list[str] = []
+        self._pyobjects: dict[str, Any] = {}
+
+    def get_count(self) -> int:
+        return len(self._gobjects)
+
+    def track(self, obj: GObject.Object, path: str | None) -> None:
+        if path is None:
+            full_path = f"({id(obj)}) {obj.__class__.__name__}"
+        else:
+            full_path = f"({id(obj)}) {path} -> {obj.__class__.__name__}"
+
+        obj.weak_ref(partial(self._gobjects.remove, full_path))
+        self._gobjects.append(full_path)
+
+        finalizer = weakref.finalize(obj, partial(self._pyobjects.pop, full_path))
+        self._pyobjects[full_path] = finalizer
+
+    def iter_pyobjects(self) -> Iterator[tuple[str, Any]]:
+        yield from self._pyobjects.items()
+
+    def iter_gobjects(self) -> Iterator[str]:
+        yield from self._gobjects
+
+    def pyobj_finalized(self) -> bool:
+        return not bool(self._pyobjects)
+
+    def gobj_finalized(self) -> bool:
+        return not bool(self._gobjects)
+
+    def get_pyobject_leaks(self) -> str:
+        def is_finalizer_ref(ref: Any) -> bool:
+            try:
+                return isinstance(ref[2][0], str)
+            except Exception:
+                return False
+
+        log_string = ""
+        for path, finalizer in self.iter_pyobjects():
+            tup = finalizer.peek()
+            if tup is None:
+                continue
+
+            log_string += f"{path} not finalized\n"
+            log_string += "References:"
+
+            for ref in gc.get_referrers(tup[0]):
+                if is_finalizer_ref(ref):
+                    continue
+                if isinstance(ref, dict):
+                    ref = cast(Any, ref)
+                    log_string += f"{pprint.pformat(ref)}\n"
+                else:
+                    log_string += f"{ref}\n"
+
+        return log_string
+
+    def get_gobject_leaks(self) -> str:
+        return "\n".join(self.iter_gobjects())
+
+
+def check_finalize(obj: GObject.Object, only_children: bool = False) -> None:
+    if "GAJIM_LEAK" not in os.environ:
+        return
+
+    logger = logging.getLogger("gajim.leak")
+    parent_obj = obj.__class__.__name__
+
+    if isinstance(obj, Gtk.Widget):
+        objs = iterate_widget_tree(obj, only_children=only_children)
+    else:
+        objs = [(obj, None)]
+
+    tracker = ObjTracker()
+
+    for _obj, path in objs:
+        tracker.track(_obj, path)
+    del objs
+
+    obj_count = tracker.get_count()
+    if not obj_count:
+        return
+
+    def check_finalized():
+        gc.collect()
+
+        if tracker.pyobj_finalized():
+            logger.info(
+                "%s (Python Object) successful finalized (total: %s)",
+                parent_obj,
+                obj_count,
+            )
+        else:
+            logger.warning(
+                "%s (Python Object) failed to finalize (total: %s)\n%s",
+                parent_obj,
+                obj_count,
+                tracker.get_pyobject_leaks(),
+            )
+
+        if tracker.gobj_finalized():
+            logger.info(
+                "%s (GObject) successful finalized (total: %s)", parent_obj, obj_count
+            )
+        else:
+            logger.warning(
+                "%s (GObject) failed to finalize (total: %s)\n%s",
+                parent_obj,
+                obj_count,
+                tracker.get_gobject_leaks(),
+            )
+
+    GLib.timeout_add_seconds(2, check_finalized)
